@@ -78,6 +78,11 @@ impl<'stmt> Rows<'stmt> {
     {
         AndThenRows { rows: self, map: f }
     }
+
+    /// Give access to the underlying statement
+    pub fn as_ref(&self) -> Option<&Statement<'stmt>> {
+        self.stmt
+    }
 }
 
 impl<'stmt> Rows<'stmt> {
@@ -151,7 +156,7 @@ where
         self.rows
             .next()
             .transpose()
-            .map(|row_result| row_result.and_then(|row| (map)(&row)))
+            .map(|row_result| row_result.and_then(|row| (map)(row)))
     }
 }
 
@@ -176,7 +181,7 @@ where
         self.rows
             .next()
             .transpose()
-            .map(|row_result| row_result.map_err(E::from).and_then(|row| (map)(&row)))
+            .map(|row_result| row_result.map_err(E::from).and_then(|row| (map)(row)))
     }
 }
 
@@ -205,7 +210,7 @@ impl<'stmt> FallibleStreamingIterator for Rows<'stmt> {
     #[inline]
     fn advance(&mut self) -> Result<()> {
         match self.stmt {
-            Some(ref stmt) => match stmt.step() {
+            Some(stmt) => match stmt.step() {
                 Ok(true) => {
                     self.row = Some(Row { stmt });
                     Ok(())
@@ -358,6 +363,12 @@ impl<'stmt> Row<'stmt> {
     }
 }
 
+impl<'stmt> AsRef<Statement<'stmt>> for Row<'stmt> {
+    fn as_ref(&self) -> &Statement<'stmt> {
+        self.stmt
+    }
+}
+
 mod sealed {
     /// This trait exists just to ensure that the only impls of `trait Params`
     /// that are allowed are ones in this crate.
